//----------------------------------------------------------------------------------
// File:   Deform.fx
// Author: Nuttapong Chentanez
// Email:  sdkfeedback@nvidia.com
//
// Deformable Physics is based on Meshless Deformations Based on Shape Matching by M. Mueller, B. Heidelberger, M. Teschner, M. Gross.
// Collision detection and response is done with cube map look up.
// All run-time computation is done on the GPU.
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//

// For rendering landscape
//

//--------------------------------------------------------------------------------------
// Constant Buffer Variables
//--------------------------------------------------------------------------------------
Texture2D<float4> txHeight;

Texture2D<float4> redTex;
Texture2D<float4> greenTex;
Texture2D<float4> blueTex;
Texture2D<float4> alphaTex;
Texture2D<float4> coverageTex;

// Amount of jittering and scaling for the textures corresponding to r, g, b, a
float2 uvJitterRed;
float uvScalingRed;
float2 uvJitterGreen;
float uvScalingGreen;
float2 uvJitterBlue;
float uvScalingBlue;
float2 uvJitterAlpha;
float uvScalingAlpha;

// Linear sampling wrap
SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

// Linear sampling clamp to border
SamplerState samHeight
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Border;
    AddressV = Border;
    BorderColor = float4(0,1,0,0);
};

// Render everything
RasterizerState renderAll 
{
	FillMode = SOLID;
	CullMode = NONE;	
};

// Constants that change only on resize
cbuffer cbChangeOnResize
{
    matrix projection;
};

// Constants that change every frame
cbuffer cbChangesEveryFrame
{
    matrix world;
    matrix view;
};

struct VS_INPUT
{
    float3 pos : POSITION;
    float2 uv : TEXCOORD0;
    
};

struct PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 position : TEXCOORD1;
    float3 normal : TEXCOORD2;
    float2 uv : TEXCOORD0;

    float2 uvRed : TEXCOORD4;
    float2 uvGreen : TEXCOORD5;
    float2 uvBlue : TEXCOORD6;
    float2 uvAlpha : TEXCOORD7;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT input, uint id:SV_VertexID )
{
    PS_INPUT output = (PS_INPUT)0;
    float4 hm = txHeight.SampleLevel(samHeight,input.uv, 0);
    
    output.position = float4(input.pos.x, hm.w, input.pos.z, 1.0f);
    output.position = mul(output.position, world );
    output.pos = mul( output.position, view );
    output.pos = mul( output.pos, projection );
    output.uv = input.uv;
    output.normal = mul(float4(hm.x, hm.y,hm.z,0.0f), world).xyz;
	output.uvRed = input.uv * uvScalingRed + uvJitterRed;
    output.uvGreen = input.uv * uvScalingGreen + uvJitterGreen;
    output.uvBlue = input.uv * uvScalingBlue + uvJitterBlue;
    output.uvAlpha = input.uv * uvScalingAlpha + uvJitterAlpha;
    
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS( PS_INPUT input) : SV_Target
{
	float3 l = normalize(float3(0.0f, 20.0f, 20.0f) - input.position.xyz);
	float3 n = normalize(input.normal);
	float4 coverage = coverageTex.Sample(samLinear, input.uv);
	
    float i = 0.2 + dot(l,n)*0.2;
    return i*(coverage.x*redTex.Sample(samLinear, input.uvRed) + 
		      coverage.y*greenTex.Sample(samLinear, input.uvGreen) + 
		      coverage.z*blueTex.Sample(samLinear, input.uvBlue) + 
		      coverage.w*alphaTex.Sample(samLinear, input.uvAlpha));
}

BlendState BlendNone{
	BlendEnable[0] = FALSE;
	RenderTargetWriteMask[0] = 0x0F;	
};

DepthStencilState DepthStencil{
    DepthEnable = TRUE;
    DepthWriteMask = ALL;
    DepthFunc = Less;

    // Setup stencil states
    StencilEnable = false;
    StencilReadMask = 0x00;
    StencilWriteMask = 0x00;


};
//--------------------------------------------------------------------------------------
technique10 Render
{
    pass P0
    {
		SetRasterizerState(renderAll);
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetBlendState(BlendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        SetDepthStencilState( DepthStencil, 0 );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}
