//----------------------------------------------------------------------------------
// File:   DeformableBody.fx
// Author: Nuttapong Chentanez
// Email:  sdkfeedback@nvidia.com
//
// Deformable Physics is based on Meshless Deformations Based on Shape Matching by M. Mueller, B. Heidelberger, M. Teschner, M. Gross.
// Collision detection and response is done with cube map look up.
// All run-time computation is done on the GPU.
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//

//------------------------------- Textures that store information about control points --------------------------------------------------------------------
// xTexH x xTexW is enough to hold information about all control points
// Sorted by number clusters that influence them
Texture2D<uint>	  objIDTex;				// I : Storing the id of the object that this control point belongs to
Texture2D<float4> xTex;					// IO: Position of control points for pingpong xTexH x xTexW, Position, w store mass 
Texture2D<float4> xTildaTex;			// IO: Intermediate position of control points xTexH x xTexW
Texture2D<float4> vTex;					// IO: For pingpong MxNxTexH x xTexW Velocity, w store alpha, if alpha == 0, means that this is a constrained node (should not move)
Texture2D<float4> gTex;					// IO: goal position for the control points xTexH x xTexW
Texture2D<float4> forceTex;				// IO: Total force acting on the control points xTexH x xTexW
Texture2D<float4> aTildaTex;			// IO: Total acceleration including collision force, external force, collision with heightmap force xTexH x xTexW 

// **  size vary, the total # of entries in the texture is equal to the sum of the number of control points in all the clusters (including duplicate)
Texture2D<uint2> gAdrTex;				// I: pointer to gValTex to fetch the goal position from, for each control points 

//------------------------------- Textures that store information about control points in each cluster ----------------------------------------------------
// xAdrTexH x xAdrTexW is enough to hold all control points in all clusters (if a control point is influence by 4 clusters, count it 4 times)
//	   xAdrTexW is divisible by number of stuffs to sum in 1 point
Texture2D<uint2> xAdrTex;				// I: Address of control points in each cluster xAdrTexH x xAdrTexW ,     
										//    XY = pointer to m_xTex
Texture2D<float4> xValTex;				// IO: position  for each cluster xAdrTexH x xAdrTexW 
										//     XYZ = XYZ of x
										//     W = mass
Texture2D<float4> pValTex;				// IO: control points' position relative to cluster's CM for each cluster xAdrTexH x xAdrTexW 
Texture2D<float4> qBarTex;				// I: Pre-computed qbar (q is included as a special case) xAdrTexH x xAdrTexW*3
										// (q_x, q_y, q_z, q_xx)
										// (q_yy, q_zz, q_xy, q_xz)
										// (q_yz, junk, junk, junk)
Texture2D<float4> gValTex;				// IO: goal position for control points in each cluster			 xAdrTexH x xAdrTexW 
Texture2D<float4> forceValTex;			// IO: Collision force act on control points in each cluster     xAdrTexH x xAdrTexW 

//------------------------------- Textures that store information about each cluster -------------------------------------------------------------------------
// cmTexH x cmTexW is enough to hold information about each clusters 
Texture2D<float4> cmTex;				// IO: cm for each cluster            cmTexH x cmTexW
										//     XYZ = XYZ of cm 
										//     W = accumulated mass
Texture2D<float4> nmaxsTex;				// IO: Negated bounding box maxs, cmTexH x cmTexW
Texture2D<float4> minsTex;				// IO: Bounding box mins, cmTexH x cmTexW
Texture2D<float4> ApqBarTex;			// IO: Apq bar for each cluster cmTexH x cmTexW*7, Apq is the 3x3 submatrix to the left
										// (Apqb11, Apqb12, Apqb13, Apqb14)
										// (Apqb15, Apqb16, Apqb17, Apqb18)
										// (Apqb21, Apqb22, Apqb23, Apqb24)
										// (Apqb25, Apqb26, Apqb27, Apqb28)
										// (Apqb31, Apqb32, Apqb33, Apqb34)
										// (Apqb35, Apqb36, Apqb37, Apqb38)
										// (Apqb19, Apqb29, Apqb39)
Texture2D<float4> AqqBarTex;			// I: Pre-computed symmetric m_AqqBar cmTexH x cmTexW*12, there are 45 independent component
										// (Aqqb11, Aqqb12, Aqqb13, Aqqb14)
										// (Aqqb15, Aqqb16, Aqqb17, Aqqb18)
										// (Aqqb19, Aqqb22, Aqqb23, Aqqb24)
										// (Aqqb25, Aqqb26, Aqqb27, Aqqb28)
										// (Aqqb29, Aqqb33, Aqqb34, Aqqb35)
										// (Aqqb36, Aqqb37, Aqqb38, Aqqb39)
										// (Aqqb44, Aqqb45, Aqqb46, Aqqb47)
										// (Aqqb48, Aqqb49, Aqqb55, Aqqb56)
										// (Aqqb57, Aqqb58, Aqqb59, Aqqb66)
										// (Aqqb67, Aqqb68, Aqqb69, Aqqb77)
										// (Aqqb78, Aqqb79, Aqqb88, Aqqb89)
										// (Aqqb99, junk, junk, junk)
Texture2D<float4> transformTex;			// IO: T=Beta*Abar + (1-Beta)*R for each cluster cmTexH x cmTexW*9
										// Store this by column, so that we can do weighted added
										// (T11, T21, T31, 0)
										// (T12, T22, T32, 0)
										// (T13, T23, T33, 0)
										// (T14, T24, T34, 0)
										// (T15, T25, T35, 0)
										// (T16, T26, T36, 0)
										// (T17, T27, T37, 0)
										// (T18, T28, T38, 0)
										// (T19, T29, T39, 0)

//------------------------------------- Texture that store information about real vertices (vertices of surface mesh for rendering)----------------------------
Texture2D<float4> posTex;				// IO: Contain real vertices' position computed as a function of control points' position
Texture2D<float4> normalTex;			// IO: Contain real vertices' normal
Texture2D<uint4> posAdrTex;				// I: Contain pointers to the 4 control points indices that affect each real vertex
Texture2D<float4> posWeightTex;			// I: Contain weight of the 4 control points that affect each real vertex

//------------------------------------- MISC textures -------------------------------------------------------------
Texture2D<float4> diffuseTex;			// Diffuse texture for mesh rendering
Texture2D<float4> heightMapTex;			// Texture of the height map, xyz contains normal and w contains the height
TextureCube<float2> cubeMapAtlasTex;	// Store uv coordinate for looking up into a 2D texture atlas of flatten cube maps 
Texture2DArray<float4> cube2DTex;		// Texture 2D array of the flatten cube map for each clusters, each slice stores several cube maps
Buffer<uint2> idToxValTexMappingBuffer; // Buffer containing the uv coordnates in the xValTex that the position for a given control vertex (indexed by ID) resides
Buffer<uint> renderTargetIDBuffer;		// Buffer containing the render target that each of cube map should be rendered to
Buffer<float2> uvBuffer;				// Buffer containing uv coordinates
Buffer<uint> uvIndexBuffer;				// UV Index buffer


#define ROUND(a) round(a)

// A constant that is added to the cube map depth so that objects will start to slow down before
// they actually penetrate 
#define DISTANCE_FUDGE_FACTOR 0.1f
#define HEIGHTMAP_FUDGE_FACTOR 0.1f
#define CLAMP_ACCELERATION_SQ 2000000.0f

// DS for no depth test and no depth write
DepthStencilState noDepthTest{
    DepthEnable = false;
    DepthWriteMask = ZERO;
    DepthFunc = Less;

    // Setup stencil states
    StencilEnable = false;
    StencilReadMask = 0x00;
    StencilWriteMask = 0x00;
};

// DS for depth test enable
DepthStencilState depthTest{
    DepthEnable = true;
    DepthWriteMask = ALL;
    DepthFunc = Less;

    // Setup stencil states
    StencilEnable = false;
    StencilReadMask = 0x00;
    StencilWriteMask = 0x00;
};

// Sampler state for wrapped mip-map bilinear interpolation
SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

// Sampler state for clamp to border mip-map bilinear
SamplerState samLinearBorder
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Border;
    AddressV = Border;
    BorderColor = float4(0,1,0,0);
};

// Sampler state for nearest sampler
SamplerState samNearest
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};

// RS for rendering everything without culling
RasterizerState renderAll 
{
	FillMode = SOLID;
	CullMode = NONE;	
    MultisampleEnable = true;
};

// BS for no blending
BlendState blendNone{
	BlendEnable[0] = FALSE;
	BlendEnable[1] = FALSE;
	RenderTargetWriteMask[0] = 0x0F;	
	RenderTargetWriteMask[1] = 0x0F;	
};

// BS for additive blending
BlendState blendAdd{
	BlendEnable[0] = TRUE;
	BlendEnable[1] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ONE;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[0] = 0x0F;	
	RenderTargetWriteMask[1] = 0x0F;	
};

// BS for max blending
BlendState blendMax{
	BlendEnable[0] = TRUE;
	BlendEnable[1] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = MAX;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ONE;
	BlendOpAlpha = MAX;
	RenderTargetWriteMask[0] = 0x0F;	
	RenderTargetWriteMask[1] = 0x0F;	
};

// BS for min blending
BlendState blendMin{
	BlendEnable[0] = TRUE;
	BlendEnable[1] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = MIN;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ONE;
	BlendOpAlpha = MIN;
	RenderTargetWriteMask[0] = 0x0F;	
	RenderTargetWriteMask[1] = 0x0F;	
};

// BS for writing alpha
BlendState writeAlphaOnly{
	BlendEnable[0] = FALSE;
	RenderTargetWriteMask[0] = 0x08;	
};

//--------------------------------------------------------------------------------------
// Constant Buffer Variables
//--------------------------------------------------------------------------------------

// Constants that never change
cbuffer cbNeverChanges
{
    matrix view;
    float twoInv7S;
    float inv3xAdrW;
    float threeXAdrW;
    uint xTexW;
    uint xAdrTexW;
    uint posTexW;
    uint cmTexW;
    float computeNormalScaleU, computeNormalScaleV;
    float3 invHeightScale;
    
    float4 clustersQuads[numClusters]; 
    float3 cubeTexOffset[numClusters]; // Contain offset into the texture2Darray map of the cube2DTex 
    uint clustersToObjects[numClusters]; // Map the cluster to the object the cluster belong to
    matrix cubeMapViewProj[6];
    
    float twoInvXTexW, twoInvXTexH; 
    
    // For using >> and & to optimize performance
    uint shift_xTexW, and_xTexW;
    uint shift_xAdrTexW, and_xAdrTexW;
    uint shift_posTexW, and_posTexW;
    uint shift_cmTexW, and_cmTexW;
    float gravity;
};

// Constants that change when windows is resized
cbuffer cbChangeOnResize
{
    matrix projection;
};

// Constants that change every frame
cbuffer cbChangesEveryFrame
{
	float h, invH;
    matrix world;
    uint startVertex;
    matrix worldView;
    matrix worldViewProj;
    matrix worldViewIT;
    float3 lightPos;
    
    float2 minsPickNC; 
    float2 maxsPickNC; 
    float3 cameraPos; 
    float3 mouseRay; 

};
cbuffer cbChangesWithUserInteraction
{
    float betas[numClusters];			// Storing betas
    float alphas[numClusters];			// Storing alpha, for each object (but numClusters >= numObjects)
    float clustersHardness[numClusters]; // The hardness of objects

    float floorHardness;
    float airDrag;						 // The coefficient to be multiplied to the velocity to slow down the motion of the objects, should be close to 1 but <= 1
    float springK; 
}

//******************************************** Debug ****************************************

struct DEBUG_GS_INPUT
{
	float3 pos1 : POSITION;
	float3 pos2 : TEXCOORD0;
	float3 pos3 : TEXCOORD1;
	//float3 pos : POSITION;
	//float3 vec : TEXCOORD0;
};
struct DEBUG_PS_INPUT
{
	float4 pos : SV_POSITION;
	float4 color : COLOR;
};

DEBUG_GS_INPUT DEBUG_VS(uint id:SV_VertexID )
{
	// Pick the control point
	DEBUG_GS_INPUT output;
	uint3 fetchPos = uint3(id & and_xTexW, id >> shift_xTexW, 0);
	output.pos1 = xTildaTex.Load(fetchPos).xyz;
	
	float4 g = gTex.Load(fetchPos);	
	output.pos2 = g.xyz / g.w;
	
	uint id2 = id % numClusters;
	uint3 fetchCMPos = uint3(id2 & and_cmTexW, id2 >> shift_cmTexW, 0);
	float4 cm = cmTex.Load(fetchCMPos);
	output.pos3 = cm.xyz / cm.w;
	return output;
}
[maxvertexcount(3)]
void DEBUG_GS( point DEBUG_GS_INPUT input[1], inout PointStream<DEBUG_PS_INPUT> lines )
{
	DEBUG_PS_INPUT pt;
	pt.pos = mul(float4(input[0].pos1, 1.0f), worldViewProj);
	pt.color = float4(1,0,0,1);
	lines.Append(pt);
	pt.pos = mul(float4(input[0].pos2, 1.0f), worldViewProj);
	pt.color = float4(0,1,0,1);
	lines.Append(pt);	
	pt.pos = mul(float4(input[0].pos3, 1.0f), worldViewProj);
	pt.color = float4(1,1,1,1);
	lines.Append(pt);	
}
float4 DEBUG_PS( DEBUG_PS_INPUT input) : SV_Target
{
    return input.color;
}
technique10 Debug
{
    pass P0
    {
		SetDepthStencilState( noDepthTest, 0 );
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		SetRasterizerState(renderAll);    	
        SetVertexShader( CompileShader( vs_4_0, DEBUG_VS() ) );        
        SetGeometryShader( CompileShader( gs_4_0, DEBUG_GS() )  );
        SetPixelShader( CompileShader( ps_4_0, DEBUG_PS() ) );
    }
}
/*
 // For debugging cube map
struct DEBUG_GS_INPUT
{
	float4 pos : POSITION;
};

struct DEBUG_PS_INPUT
{
	float4 pos : SV_POSITION;
};

DEBUG_GS_INPUT DEBUG_VS(uint id:SV_VertexID) {
	DEBUG_GS_INPUT output;
	output.pos = float4(0.0f, 0.0f, 0.0f, 0.0f);
	return output;
}

[maxvertexcount(4)]
void DEBUG_GS( point DEBUG_GS_INPUT input[1], inout TriangleStream<DEBUG_PS_INPUT> triangles )
{
	DEBUG_PS_INPUT output;
	output.pos = float4(-1.0f, -1.0f, 0.0f, 1.0f);
	triangles.Append(output);    
	output.pos = float4(-1.0f, 1.0f, 0.0f, 1.0f);
	triangles.Append(output);    
	output.pos = float4(1.0f, -1.0f, 0.0f, 1.0f);
	triangles.Append(output);    
	output.pos = float4(1.0f, 1.0f, 0.0f, 1.0f);
	triangles.Append(output); 
	triangles.RestartStrip();   
}
float4 DEBUG_PS( DEBUG_PS_INPUT input) : SV_Target
{
    return float4(cube2DTex.Load(int4(input.pos.x, ((uint)input.pos.y) % 32, ((uint)input.pos.y) / 32, 0)).xyz*0.5f, 1.0f);
}
technique10 Debug
{
    pass P0
    {
		SetDepthStencilState( noDepthTest, 0 );
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		SetRasterizerState(renderAll);    	
        SetVertexShader( CompileShader( vs_4_0, DEBUG_VS() ) );        
        SetGeometryShader( CompileShader( gs_4_0, DEBUG_GS() ) );
        SetPixelShader( CompileShader( ps_4_0, DEBUG_PS() ) );
    }
}
*/

//******************************************** Picked Points Rendering *****************************************
struct PICKEDPOINTS_RENDER_VS_INPUT {
	uint2 adr : TEXCOORD0;
};

struct PICKEDPOINTS_RENDER_PS_INPUT{
	float4 pos : SV_POSITION;
	float4 color : COLOR;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Fetch the control point position and then output.
//--------------------------------------------------------------------------------------

PICKEDPOINTS_RENDER_PS_INPUT PICKEDPOINTS_RENDER_VS(PICKEDPOINTS_RENDER_VS_INPUT input) {
	PICKEDPOINTS_RENDER_PS_INPUT output;	
	
	// Fetch the control point position
	float3 position = xTex.Load(uint3(input.adr, 0)).xyz;
	
	output.pos = mul(float4(position, 1.0f), worldViewProj);
	output.color = float4(0.0f, 1.0f, 0.0f, 1.0f);
	return output;
};

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Just pass through the color
//--------------------------------------------------------------------------------------
float4 PICKEDPOINTS_RENDER_PS(PICKEDPOINTS_RENDER_PS_INPUT input) : SV_Target{
	return input.color;
}

technique10 PickedPointsRender
{
    pass P0
    {
		// No Depth test
    	SetDepthStencilState( noDepthTest, 0 );
    	
    	// No Blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, PICKEDPOINTS_RENDER_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PICKEDPOINTS_RENDER_PS() ));
    }
}




//******************************************** Mesh Rendering *****************************************

struct MESH_RENDER_GS_INPUT
{
    float4 position : POSITION;
    float3 normal : NORMAL;    
    float  JUNK1 : TEXCOORD2;
    float3 pos : TEXCOORD1;
    float JUNK2 : TEXCOORD3;
    float2 uv : TEXCOORD0;
};
struct MESH_RENDER_PS_INPUT
{
    float4 position : SV_POSITION;
    float3 normal : NORMAL;    
    float  JUNK1 : TEXCOORD2;
    float3 pos : TEXCOORD1;
    float JUNK2 : TEXCOORD3;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader:
// Input:  vertex ID
// Output: position in NC
//         position in world coordinate
//		   normal in world coordinate
//		   uv just (0.0f, 0.0f) in this case 
// Functionality: 
//		   Fetch appropriate vertex position and transform with worldviewproj
//         Fetch normal and transform with worldViewIT
//--------------------------------------------------------------------------------------
MESH_RENDER_GS_INPUT MESH_RENDER_VS(uint id:SV_VertexID )
{
    MESH_RENDER_GS_INPUT output = (MESH_RENDER_PS_INPUT)0;
    id += startVertex;
    uint3 fetchPos = uint3(id & and_posTexW, id >> shift_posTexW, 0);
    output.uv = float2(0.0f, 0.0f);
    output.normal = mul(float4(normalize(normalTex.Load(fetchPos).xyz), 0), worldViewIT).xyz;
    output.position = float4(posTex.Load(fetchPos).xyz, 1.0f);
    output.pos = mul(output.position, worldView).xyz;
    output.position = mul(output.position, worldViewProj);
    output.JUNK1 = 1.0f;
    output.JUNK2 = 1.0f;
    return output;
}

//--------------------------------------------------------------------------------------
// Geometry Shader:
// Input:  triangle, primitive ID
// Output: triangle with correct uv coordinate
// Functionality:
//		   Take the primitive ID, multiply by 3 and look into uv index buffer and then 
//		   fetch the correct uv coordinate. This way, we can have vertices with multiple 
//		   uv coordinates
//--------------------------------------------------------------------------------------
[maxvertexcount(3)]
void MESH_RENDER_GS( triangle MESH_RENDER_GS_INPUT input[3], inout TriangleStream<MESH_RENDER_PS_INPUT> triangles, uint id:SV_PrimitiveID  )
{	
	id *= 3;
	input[0].uv = uvBuffer.Load(uvIndexBuffer.Load(id));
	input[1].uv = uvBuffer.Load(uvIndexBuffer.Load(id+1));
	input[2].uv = uvBuffer.Load(uvIndexBuffer.Load(id+2));	
	
	triangles.Append(input[0]);
	triangles.Append(input[1]);
	triangles.Append(input[2]);
}

//--------------------------------------------------------------------------------------
// Pixel Shader:
// Functionality:
//			Compute light vector, normalize normal, view vector, half vector and compute 
//		    the lighting
//--------------------------------------------------------------------------------------
float4 MESH_RENDER_PS( MESH_RENDER_PS_INPUT input) : SV_Target
{
	float3 l = normalize(lightPos-input.pos);
	float3 n = normalize(input.normal);
	float3 v = float3(0.0f, 0.0f, -1.0f);
	float3 h = normalize(l+v);
	return float4(diffuseTex.Sample(samLinear, input.uv).xyz*(0.2 + dot(l,n)*0.8), 1.0f);	
}

technique10 MeshRender
{
    pass P0
    {
		// With depth test
		SetDepthStencilState( depthTest, 0 );
		
		// No blending 
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render eveything
		SetRasterizerState(renderAll);    	
		
        SetVertexShader( CompileShader( vs_4_0, MESH_RENDER_VS() ) );
        SetGeometryShader( CompileShader( gs_4_0, MESH_RENDER_GS() ) );
        SetPixelShader( CompileShader( ps_4_0, MESH_RENDER_PS() ) );
    }
}

//******************************************** Point RENDER *****************************************
//*************************************** For debugging, can be ignored ****************************
struct POINT_RENDER_GS_INPUT
{
    float4 pos : SV_POSITION;
};
struct POINT_RENDER_PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
};

//--------------------------------------------------------------------------------------
// Vertex Shader:
// Input:  vertex id
// Output: vertex position in NC
// Functionality:
//		   Just fetch the vertex position and transform with worldViewProj
//--------------------------------------------------------------------------------------

POINT_RENDER_GS_INPUT POINT_RENDER_VS(uint id:SV_VertexID )
{
	// Pick the control point
	POINT_RENDER_GS_INPUT output;
	uint3 fetchPos = uint3(id & and_xTexW, id >> shift_xTexW, 0);
	output.pos = mul(float4(xTex.Load(fetchPos).xyz, 1.0f), worldViewProj);
	return output;
}

//--------------------------------------------------------------------------------------
// Geometry Shader:
// Input: a point
// Output: a point
// Functionality:
//     Check if the point is in the rectangle with bottom left at minsPickNC and top right 
//	   at maxsPickNC. If so, color it red otherwise, color it white.
//--------------------------------------------------------------------------------------
[maxvertexcount(1)]
void POINT_RENDER_GS(point POINT_RENDER_GS_INPUT input[1], inout PointStream<POINT_RENDER_PS_INPUT> pointStream) {
	float3 nc = input[0].pos.xyz / input[0].pos.w;
	POINT_RENDER_PS_INPUT output;
	// Check if the vertex is in the range that I should mark as being picked
    output.pos = input[0].pos;
	output.color = float4(1.0f, 1.0f, 1.0f, 1.0f);	
	if ( (nc.x >= minsPickNC.x) && (nc.x <= maxsPickNC.x) && 
		 (nc.y >= minsPickNC.y) && (nc.y <= maxsPickNC.y)) {
		 output.color = float4(1.0f, 0.0f, 0.0f, 1.0f);	
		 pointStream.Append(output);				 	 
	}	
	
}
//--------------------------------------------------------------------------------------
// Pixel Shader:
// Functionality:
//		Just output the color;
//--------------------------------------------------------------------------------------
float4 POINT_RENDER_PS( POINT_RENDER_PS_INPUT input) : SV_Target
{
    return input.color;
}

technique10 PointRender
{
    pass P0
    {
		// With depth test
		SetDepthStencilState( depthTest, 0 );
		
		// No blending 
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render eveything
		SetRasterizerState(renderAll);    	

        SetVertexShader( CompileShader( vs_4_0, POINT_RENDER_VS() ) );
        SetGeometryShader(CompileShader( gs_4_0, POINT_RENDER_GS() ) );
        SetPixelShader( CompileShader( ps_4_0, POINT_RENDER_PS() ) );
    }
}
//******************************************** Compute XVAL *****************************************
// Compute XVal for each points in each cluster
struct COMPUTE_XVAL_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_XVAL_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader:
// Functionality:
//		Just pass through. The input will be the corners of the quads that cover the 
//      whole xValTex
//--------------------------------------------------------------------------------------
COMPUTE_XVAL_PS_INPUT COMPUTE_XVAL_VS( COMPUTE_XVAL_VS_INPUT input)
{
    COMPUTE_XVAL_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader:
// Functionality:
//      Fetch the xAdrTex, to get the address of the control point we need to fetch the
//      position from. Then we fetch the position of that control point.
//      The resulting xValTex will now contain the position of the control points
//      that belong to the clusters. This pass is done because we want to do the 
//      summation over all the control points in a cluster, without having to do
//      a dependent texture fetch every time,
//--------------------------------------------------------------------------------------
float4 COMPUTE_XVAL_PS( COMPUTE_XVAL_PS_INPUT input) : SV_Target
{
	int2 uvi2 = xAdrTex.Load(int3(ROUND(input.uv),0));
	int3 uvi = int3( uvi2, 0);
    return xTex.Load(uvi);   
}

technique10 ComputeXVal
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    
		
		// No blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_XVAL_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_XVAL_PS() ) );
    }
}

//******************************************** Compute CM for each cluster *****************************************
// Compute cm for each cluster
struct COMPUTE_CM_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_CM_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader:
// Functionality:
//		Just pass through. There will be several points that are rendered at a position
//      in the cmTex, corresponding to a cluster. The number of points depends on
//      how many control points are in that cluster. Each of the point basically
//      sum up the position of many control points that belong to that cluster 
//      weighted by their masses.
//--------------------------------------------------------------------------------------
COMPUTE_CM_PS_INPUT COMPUTE_CM_VS( COMPUTE_CM_VS_INPUT input)
{
    COMPUTE_CM_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//      Fetch the xValTex several time to get the positions and mass of some of the control 
//      points that are members of the cluster. Sum them weighted by the mass and output.
//      The position of these control points are stored in adjacent texels in the xValTex.
//--------------------------------------------------------------------------------------
float4 COMPUTE_CM_PS( COMPUTE_CM_PS_INPUT input) : SV_Target
{	
	int3 fetchPos = int3(ROUND(input.uv),0);
	float4 tmp = xValTex.Load(fetchPos);	
	float4 acc = float4(tmp.xyz*tmp.w, tmp.w);
	[unroll(sumPerPixel)] for (int i = 1; i < sumPerPixel; i++) {
		fetchPos.x++;
		tmp = xValTex.Load(fetchPos);
		acc += float4(tmp.xyz*tmp.w, tmp.w);
	}	
	return acc;    
}

technique10 ComputeCM
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Do 32bit additive blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		
		// Render eveything
		SetRasterizerState(renderAll);    
		
		SetVertexShader( CompileShader( vs_4_0, COMPUTE_CM_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_CM_PS() ) );
    }
}

// Computing mins and maxs
void COMPUTE_MINS_NMAXS_PS( COMPUTE_CM_PS_INPUT input, out float3 accMins : SV_Target0, out float3 accNMaxs : SV_Target1) 
{
	int3 fetchPos = int3(ROUND(input.uv),0);
	float4 tmp = xValTex.Load(fetchPos);	
		
	accMins = lerp(tmp.xyz, float3(1e20, 1e20, 1e20), 1.0f-any(tmp.w));
	accNMaxs = lerp(-tmp.xyz, float3(1e20, 1e20, 1e20), 1.0f-any(tmp.w));
	// Unroll the loop for performance improvement
	[unroll(sumPerPixel-1)] for (int i = 1; i < sumPerPixel; i++) {
		fetchPos.x++;
		tmp = xValTex.Load(fetchPos);
		accMins = min(accMins, lerp(tmp.xyz, float3(1e20, 1e20, 1e20), 1.0f-any(tmp.w)));
		accNMaxs = min(accNMaxs, lerp(-tmp.xyz, float3(1e20, 1e20, 1e20), 1.0f-any(tmp.w)));
	}
}

technique10 ComputeMinsNMaxs
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

 		// Do the min blending, so as to compute the min
		SetBlendState(blendMin, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_CM_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_MINS_NMAXS_PS() ) );
    }
}
//******************************************** Pass Compute pVal *****************************************
// Compute pVal for each vertex in each cluster using alpha blending add by -cm 
struct COMPUTE_PVAL_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_PVAL_PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 cm : COLOR0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Fetch the center of mass and pass it as a vertex attribute
//			 There will be one quad per cluster that will be drawn to the pValTex
//			 Each quad will cover all control points in a cluster.
//			 For each of the cluster, pVal = xVal - cm
//			 Note that cm is the same for all the fragments in a quad, so we
//           fetch it in VS and pass as an attribute, so don't need to fetch
//			 cm very often
//--------------------------------------------------------------------------------------

COMPUTE_PVAL_PS_INPUT COMPUTE_PVAL_VS( COMPUTE_PVAL_VS_INPUT input)
{
    COMPUTE_PVAL_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.cm = cmTex.Load(int3(input.uv.xy, 0));
    output.cm.xyz /= output.cm.w;
    output.cm.w = 0.0f;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Fetch the xValTex then subtract off by the center of mass and output
//           as pValTex
//--------------------------------------------------------------------------------------
float4 COMPUTE_PVAL_PS( COMPUTE_PVAL_PS_INPUT input) : SV_Target 
{
	return xValTex.Load(int3(input.pos.xy, 0)) - input.cm;
}
technique10 ComputePVal
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// No Blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render eveything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_PVAL_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_PVAL_PS() ) );
    }
}

//******************************************** Pass Compute Cube Maps *****************************************
//	Basically, fetch the center of mass, fetch cluster' vertex positions then project onto the 6 faces of cube maps	


struct COMPUTE_CUBEMAPS_GS_INPUT
{
	float4 pos0 : TEXCOORD0;
	float4 pos1 : TEXCOORD1;
	float4 pos2 : TEXCOORD2;
	float4 pos3 : TEXCOORD3;
	float4 pos4 : TEXCOORD4;
	float4 pos5 : TEXCOORD5;
};
struct COMPUTE_CUBEMAPS_PS_INPUT
{
	float4 pos : SV_POSITION;
	float3 position : TEXCOORD0; 
	uint3 JUNK1 : TEXCOORD1;		
	uint rtIndex : SV_RenderTargetArrayIndex;
	
	uint3 JUNK2 : TEXCOORD2;
	uint vpIndex : SV_ViewportArrayIndex;
	
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality: 
//			 Just load the pValTex based on the id and then output, this is the 
//           vertex position relative to the center of mass of the cluster in consideration
//--------------------------------------------------------------------------------------

COMPUTE_CUBEMAPS_GS_INPUT COMPUTE_CUBEMAPS_VS(uint id:SV_VertexID ) {
	// Fetch pval
	COMPUTE_CUBEMAPS_GS_INPUT output;
	float4 tmpPos = pValTex.Load(int3(idToxValTexMappingBuffer.Load(id), 0));
	output.pos0 = mul(tmpPos, cubeMapViewProj[0] );
	output.pos1 = mul(tmpPos, cubeMapViewProj[1] );
	output.pos2 = mul(tmpPos, cubeMapViewProj[2] );
	output.pos3 = mul(tmpPos, cubeMapViewProj[3] );
	output.pos4 = mul(tmpPos, cubeMapViewProj[4] );
	output.pos5 = mul(tmpPos, cubeMapViewProj[5] );
	return output;
}


//--------------------------------------------------------------------------------------
// Geometry Shader
// Functionality:
//			 Output 6 triangle each go to different view port of a render target
//           The render target is picked based on which cluster the triangle belong to 
//           The 6 triangles correspond to when we view the object from the center of mass
//           in the +x, -x, +y, -y, +z, -z direction
//--------------------------------------------------------------------------------------

[maxvertexcount(18)] void COMPUTE_CUBEMAPS_GS(triangle COMPUTE_CUBEMAPS_GS_INPUT input[3], inout TriangleStream<COMPUTE_CUBEMAPS_PS_INPUT> triangleStream, uint id : SV_PrimitiveID) {
	COMPUTE_CUBEMAPS_PS_INPUT output;
	
	output.JUNK1 = uint3(0,0,0);
	output.JUNK2 = uint3(0,0,0);
	// Load the render target ID1
	output.rtIndex = renderTargetIDBuffer.Load(id);
	
	output.vpIndex = 0;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos0;
        output.position = output.pos.xyz;
        triangleStream.Append( output );
         
    }
    triangleStream.RestartStrip();		
		
	output.vpIndex = 1;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos1;
        output.position = output.pos.xyz;
        triangleStream.Append( output );         
    }
    triangleStream.RestartStrip();	
    
	output.vpIndex = 2;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos2;
        output.position = output.pos.xyz;
        triangleStream.Append( output );
         
    }
    triangleStream.RestartStrip();		
    	
	output.vpIndex = 3;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos3;
        output.position = output.pos.xyz;
        triangleStream.Append( output );
         
    }
    triangleStream.RestartStrip();		

	output.vpIndex = 4;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos4;
        output.position = output.pos.xyz;
        triangleStream.Append( output );
         
    }
    triangleStream.RestartStrip();		

	output.vpIndex = 5;
	for( int vertex = 0; vertex < 3; vertex++ ) {	
		output.pos = input[vertex].pos5;
        output.position = output.pos.xyz;
        triangleStream.Append( output );
         
    }
    triangleStream.RestartStrip();		
	
	/*
	for (int face = 0; face < 6; face++) {		
		
		// Select the correct view port 
		output.vpIndex = face;		
		
        for( int vertex = 0; vertex < 3; vertex++ )
        {
			// Transform with the view-proj matrix of the corresponding viewport
            output.pos = mul( input[vertex].pos, cubeMapViewProj[face] );
            output.position = output.pos.xyz;
            triangleStream.Append( output );
            
        }
        triangleStream.RestartStrip();		
	}
	*/
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Compute the distance from the center of mass and output as color
//           At the time of writing this demo, the depth cube map is not supported 
//			 in DX10 yet, so we need to use max blending instead but it should
//           be straight forward to use depth test instead of the max blending.
//--------------------------------------------------------------------------------------

float4 COMPUTE_CUBEMAPS_PS( COMPUTE_CUBEMAPS_PS_INPUT input) : SV_Target {
	float dis = sqrt(dot(input.position, input.position));
	return float4(dis, dis, dis, dis);
}

technique10 ComputeCubeMaps
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Render everything
		SetRasterizerState(renderAll);    

		// Use max blending
		SetBlendState(blendMax, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_CUBEMAPS_VS() ) );
        SetGeometryShader(CompileShader( gs_4_0, COMPUTE_CUBEMAPS_GS() ));
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_CUBEMAPS_PS() ) );
    }
}

//******************************************** Pass Compute Potential Colliding Pair *****************************************

struct COMPUTE_COLLISION_PAIRS_GS_INPUT
{
	uint2 pos : TEXCOORD0;
};
struct COMPUTE_COLLISION_PAIRS_GS_OUTPUT
{
    uint2 pos : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Just turn the id into pair (i, j) where 0<= i,j <numClusters 
//--------------------------------------------------------------------------------------
COMPUTE_COLLISION_PAIRS_GS_INPUT COMPUTE_COLLISION_PAIRS_VS( COMPUTE_COLLISION_PAIRS_GS_INPUT input )
{    
    
    return input;
}

//--------------------------------------------------------------------------------------
// Geometry Shader
// Input: pair (i,j)
// Output: pair (i,j)
// Functionality:
//           Check if bounding box of cluster i intersects with cluster j or not
//           If so, output the pair (i,j). Otherwise, output nothing.
//           Also, we want to make sure that if cluster i come from the same object
//           as cluster j, we ignore too
//--------------------------------------------------------------------------------------
[maxvertexcount(2)]
void COMPUTE_COLLISION_PAIRS_GS( point COMPUTE_COLLISION_PAIRS_GS_INPUT input[1], inout PointStream<COMPUTE_COLLISION_PAIRS_GS_OUTPUT> points)
{
    // Find out what object each cluster belong to 
	// They belong to different object
	// Compute the position where we need to fetch the bbox from 
	uint3 fetchPosX = uint3(input[0].pos.x & and_cmTexW, input[0].pos.x >> shift_cmTexW, 0);
	uint3 fetchPosY = uint3(input[0].pos.y & and_cmTexW, input[0].pos.y >> shift_cmTexW, 0);
	
	// fetch the bboxs
	float3 minsX = minsTex.Load(fetchPosX);
	float3 minsY = minsTex.Load(fetchPosY);
	float3 maxsX = -nmaxsTex.Load(fetchPosX);
	float3 maxsY = -nmaxsTex.Load(fetchPosY);
	
	float3 extent = min(maxsX, maxsY) - max(minsX, minsY);

	// See if the extent is a non empty interval
	if ((extent.x > 0.0f) && (extent.y > 0.0f) && (extent.z > 0.0f)) {
	
		// If so, output it
		points.Append(input[0]);
		uint t = input[0].pos.x;
		input[0].pos.x = input[0].pos.y;
		input[0].pos.y = t;
		points.Append(input[0]);
	}
    
}

// Stream out to a buffer, which later will be bound as an input vertex buffer
GeometryShader gsStreamOutComputePotentialCollisionPairs = ConstructGSWithSO( CompileShader( gs_4_0,  COMPUTE_COLLISION_PAIRS_GS() ), "TEXCOORD0.xy" );

technique10 ComputePotentialCollisionPairs
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_COLLISION_PAIRS_VS() ) );
        SetGeometryShader( gsStreamOutComputePotentialCollisionPairs );
               
        // No pixel shader
        SetPixelShader( NULL );       
                
    }
}

//******************************************** Pass Compute Collision Force *********************
struct COMPUTE_COLLISION_FORCE_VS_INPUT
{
	uint2 pos : TEXCOORD0;
};
struct COMPUTE_COLLISION_FORCE_GS_INPUT
{
	float4 quad : TEXCOORD0;
	float4 anotherObjCM : TEXCOORD1; // XYZ contains the xyz of the colliding object
	float3 anotherObjCubeTexOffset: TEXCOORD2; // XY contains offset into the cube2D Texture, Z contains the index into the texture array
	float JUNK : TEXCOORD4;
	float hardness : TEXCOORD3; // Coefficient of collision force
	
};
struct COMPUTE_COLLISION_FORCE_PS_INPUT
{
	float4 pos : SV_POSITION;
	float4 anotherObjCM : TEXCOORD1; // XYZ contains the xyz of the colliding object
	float3 anotherObjCubeTexOffset: TEXCOORD2; // XY contains offset into the cube2D Texture, Z contains the index into the texture array
	float JUNK : TEXCOORD4;
	float hardness : TEXCOORD3; // Coefficient of collision force
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Input:  pair (i,j) 
// Output: information enough for creating a quad that cover all control points in cluster i
// Functionality:
//           Obtain a quad that cover all the control points in cluster i
//           Fetch cm of cluster j,
//           Obtain the offset into the texture2Darray that contains cube map for the cluster j
//           Compute the hardness of this collision event by taking the max between
//           the hardness of i and j
//--------------------------------------------------------------------------------------

COMPUTE_COLLISION_FORCE_GS_INPUT COMPUTE_COLLISION_FORCE_VS( COMPUTE_COLLISION_FORCE_VS_INPUT input )
{
	COMPUTE_COLLISION_FORCE_GS_INPUT output;
	output.quad = clustersQuads[input.pos.x];
	output.anotherObjCM = cmTex.Load(int3(input.pos.y & and_cmTexW, input.pos.y >> shift_cmTexW, 0));
	output.anotherObjCM.xyz /= output.anotherObjCM.w;
	output.anotherObjCubeTexOffset = cubeTexOffset[input.pos.y];
	output.hardness = max(clustersHardness[input.pos.x], clustersHardness[input.pos.y]);
	output.JUNK = 1.0f;
	return output;
}


//--------------------------------------------------------------------------------------
// Geometry Shader
// Input: a point
// Output: a quad
// Functionality:
//           Turn the point into 2 triangles (a quad) that covers the portion
//           of texture forceValTex that correspond to all control points of cluster i
//           Also fetch the cm of cluster j and pass as a vertex attribute
//--------------------------------------------------------------------------------------

[maxvertexcount(4)]
void COMPUTE_COLLISION_FORCE_GS( point COMPUTE_COLLISION_FORCE_GS_INPUT input[1], inout TriangleStream<COMPUTE_COLLISION_FORCE_PS_INPUT> triangles) {
	// Need to output quad that cover all particles of cluster pos.x, and check collision again cluster pos.y
	COMPUTE_COLLISION_FORCE_PS_INPUT output;
	output.JUNK = 1.0f;
	output.anotherObjCM = input[0].anotherObjCM;
	output.anotherObjCubeTexOffset = input[0].anotherObjCubeTexOffset;
	output.hardness = input[0].hardness;
		
	output.pos = float4(input[0].quad.x, input[0].quad.y, 0.0f, 1.0f);
	triangles.Append(output);
	
	output.pos = float4(input[0].quad.x, input[0].quad.w, 0.0f, 1.0f);
	triangles.Append(output);
	
	output.pos = float4(input[0].quad.z, input[0].quad.y, 0.0f, 1.0f);
	triangles.Append(output);
	
	output.pos = float4(input[0].quad.z, input[0].quad.w, 0.0f, 1.0f);
	triangles.Append(output);			
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Fetch depth from the cube map of cluster j, add with a fudge factor
//			 then compute the distance of control points in cluster i to the cm of cluster j
//           If found to penetrate, apply force in the radial direction to push the
//           control points outward
//--------------------------------------------------------------------------------------

float4 COMPUTE_COLLISION_FORCE_PS( COMPUTE_COLLISION_FORCE_PS_INPUT input ) : SV_Target
{
	// Obtain position of a control point of cluster i
    float3 curPos = xValTex.Load(int3(input.pos.xy, 0));
    
    // Compute vector from the control point to the cm of cluster j
    float3 vec = curPos-input.anotherObjCM.xyz;
    
    // Compute the distance of the control point to the cm cluster j
    float dis = max(0.01f, sqrt(dot(vec, vec)));
    
    // Look up the cube map atlas in the direction dis
    float3 offset = float3(cubeMapAtlasTex.Sample(samLinear, vec), 0.0f);
    
    // Add offset so as to fetch the cube map of cluster j
    float3 cube2DTexFetchAdr = offset+input.anotherObjCubeTexOffset;
    
    // Fetch the cube map of cluster j 
    float distanceFromCM = cube2DTex.Sample(samLinear, cube2DTexFetchAdr);
    
    // Add distance with the fudge factor
    distanceFromCM += DISTANCE_FUDGE_FACTOR;
    
    // Check if the control point penetrate through the cube map or not
    if (dis < distanceFromCM) {
    
		// If so, add a radial force proportional to the penetration depth
		return float4((vec / dis) * (distanceFromCM - dis)*input.hardness, 1.0f);				
    } else {
		// Otherwise, ignore
		return float4(0.0f, 0.0f, 0.0f, 1.0f);
    }
}

technique10 ComputeCollisionForce
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Do additive blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
				
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_COLLISION_FORCE_VS() ));
        SetGeometryShader( CompileShader( gs_4_0, COMPUTE_COLLISION_FORCE_GS() ));
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_COLLISION_FORCE_PS() ));
    }
}
//******************************************** Pass Compute the ApqBar *****************************************
// Compute ApqBar for each cluster

struct COMPUTE_APQBAR_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_APQBAR_GS_INPUT
{
    float4 pos : SV_POSITION;
    float4 uv : TEXCOORD0;
};
struct COMPUTE_APQBAR_PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader:
// Functionality:
//			 Just pass through
//--------------------------------------------------------------------------------------
COMPUTE_APQBAR_GS_INPUT COMPUTE_APQBAR_VS( COMPUTE_APQBAR_VS_INPUT input)
{
    COMPUTE_APQBAR_GS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv;
    return output;
}


//--------------------------------------------------------------------------------------
// Geometry Shader:
// Input: 1 point
// Output: 7 point
// Functionality:
//			Sum the m p qBar^T of several control points 
//          Conceptually similar to computing CM, however, we need to fetch p from pValTex
//          then qBar from qBarTex then compute m p qBar^T and sum them together
//          Since a pixel can have at most 4 components and we want to output 27 values, 
//          as ApqBar is a 3x9 matrix, we need to output 7 adjacent fragments
//
//--------------------------------------------------------------------------------------
[maxvertexcount(7)]
void COMPUTE_APQBAR_GS( point COMPUTE_APQBAR_GS_INPUT input[1], inout PointStream<COMPUTE_APQBAR_PS_INPUT> points)
{
    COMPUTE_APQBAR_PS_INPUT output; 
    int4 uv = input[0].uv;
    
    // Compute the position in the pValTex to start fetching p of the first control point 
    int3 pValPos = int3(uv.xy, 0);  
    
    // Compute the position in the qBarTex to start fetching qBar of the first control point
    int3 qBarPos = pValPos;			
    qBarPos.x *= 3;					
    
    // Initialize the accumulation temporary variables to 0
    float4 acc0 = float4(0.0f,0.0f,0.0f,0.0f),
    	   acc1 = float4(0.0f,0.0f,0.0f,0.0f),
    	   acc2 = float4(0.0f,0.0f,0.0f,0.0f), 
    	   acc3 = float4(0.0f,0.0f,0.0f,0.0f), 
    	   acc4 = float4(0.0f,0.0f,0.0f,0.0f), 
    	   acc5 = float4(0.0f,0.0f,0.0f,0.0f), 
    	   acc6 = float4(0.0f,0.0f,0.0f,0.0f);
    	   
	float4 pVal;
	
	// Unroll the loop for performance
	[unroll(sumPerPixel)] for (int i = 0; i < sumPerPixel; i++) {
			
		// Fetch the pVal
		pVal = pValTex.Load(pValPos);
		
		// Fetch the qBar, need 3 fetch because qBar has 9 components, and also multiply by mass stored in pVal.a
		
		float4 mqBar0 = qBarTex.Load(qBarPos) * pVal.a;  
		qBarPos.x++;
		float4 mqBar1 = qBarTex.Load(qBarPos) * pVal.a;
		qBarPos.x++;
		float mqBar2 = qBarTex.Load(qBarPos) * pVal.a;
		qBarPos.x++;

		// Compute the products and accumulates
		acc0 += pVal.x * mqBar0;
		acc1 += pVal.x * mqBar1;

		acc2 += pVal.y * mqBar0;
		acc3 += pVal.y * mqBar1;

		acc4 += pVal.z * mqBar0;
		acc5 += pVal.z * mqBar1;
		
		// Special for this fragment because it stores the last column of ApqBar
		acc6 += mqBar2 * pVal;
						
		pValPos.x++;				
	}        
	
	// Generate 7 fragments each with different color
	output.pos = input[0].pos;
	output.color = acc0;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc1;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc2;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc3;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc4;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc5;
	points.Append(output);

	output.pos.x += twoInv7S;
	output.color = acc6;
	points.Append(output);
	
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Just output the color 
//--------------------------------------------------------------------------------------
float4 COMPUTE_APQBAR_PS( COMPUTE_APQBAR_PS_INPUT input) : SV_Target
{
	return input.color;
}
technique10 ComputeApqBar
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Do additive blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_APQBAR_VS() ));
        SetGeometryShader( CompileShader( gs_4_0, COMPUTE_APQBAR_GS() ));
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_APQBAR_PS() ));
    }
}

//******************************************** Pass compute the transformation matrix *****************************************
// Compute transformation matrix,  T = (1-beta)*R + beta*Abar for each cluster 

struct COMPUTE_TRANSFORM_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_TRANSFORM_GS_INPUT
{
    float4 pos : SV_POSITION;
    float4 uv : TEXCOORD0;
    float beta : TEXCOORD1;
};
struct COMPUTE_TRANSFORM_PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//			 Just pass through, basically, except that we isolate beta out
//           The input will just be a bunch of points, one point per cluster
//--------------------------------------------------------------------------------------
COMPUTE_TRANSFORM_GS_INPUT COMPUTE_TRANSFORM_VS( COMPUTE_TRANSFORM_VS_INPUT input)
{
    COMPUTE_TRANSFORM_GS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv;
    output.beta = betas[ROUND(input.pos.w)];
    return output;
}

//--------------------------------------------------------------------------------------
// Jacobi rotation function
// This function will zero out the (j, k) and (k, j) entry of a symmetric matrix mat
// by pre and post multiplying it with a rotation matrix and its transpose,
// outmat = R^T inmat R,        outjmat = injmat*R
// where R is a rotation matrix that will zero out the two entries
//--------------------------------------------------------------------------------------
void Jacobi(inout float3x3 mat, inout float3x3 jmat, in int j, in int k) {
	// First, check if entries (j,k) is too small or not, if so, do nothing
	if (abs(mat[j][k]) > 1e-20) {
		// This is just some math to figure out cosine and sine necessary to zero out the two entries
		float tau = (mat[j][j]-mat[k][k])/(2.0f*mat[j][k]);
		float t = sign(tau) / (abs(tau) + sqrt(1 + tau*tau));

		float c = 1/sqrt(1+t*t);
		float s = c*t;
		
		// Build the rotation matrix
		float3x3 R = {1.0f, 0.0f, 0.0f,
					  0.0f, 1.0f, 0.0f,
					  0.0f, 0.0f, 1.0f};
		R[j][j] = c;
		R[k][k] = c;
		R[j][k] = -s;
		R[k][j] = s;
	
		// Multiply to jmat
		jmat = mul(jmat, R);

		// Post multiply to mat
		mat = mat*R;
		
		// Transpose it
		R[j][k] = s;
		R[k][j] = -s;
		
		// Pre multiply to mat
		mat = R*mat;
	}
}

//--------------------------------------------------------------------------------------
// Compute the best fit rotation
// By diagonalizing A^tA, A^tA = J^t D J
// Please see the presentation slide for math detail
//--------------------------------------------------------------------------------------
float3x3 ComputeOptimumRotation(in float3x3 A) {
	float3x3 jmat = {1.0f, 0.0f, 0.0f,
				  0.0f, 1.0f, 0.0f,
				  0.0f, 0.0f, 1.0f};
	float3x3 mat = mul(transpose(A), A);

	// Do 5 iterations of Jacobi rotation
	[unroll(5)] for (int i = 0; i < 5; i++) {
		Jacobi(mat, jmat, 0, 1);
		Jacobi(mat, jmat, 0, 2);
		Jacobi(mat, jmat, 1, 2);
	}	
	// A^tA == jmat^t mat jmat

	// OptimumR = A jmat^t sqrt(1/mat) jmat
	float3x3 optimumR = transpose(mul(A, mul(	transpose(jmat), float3x3(jmat[0] / sqrt(mat[0][0]), 
							  			                      jmat[1] / sqrt(mat[1][1]), 
											                  jmat[2] / sqrt(mat[2][2])))));											                  
     const int first = 1, 
		second = 2, 
		third = 0;

	// Compute the cross product of the first 2 vectors to get the third basis
	// Then recompute the second vectors by crossing the third and the first one
	// This will also gaurantee that we have a right handed orientation 
    optimumR[first] = normalize(optimumR[first]);
    optimumR[third] = normalize(cross(optimumR[first], optimumR[second]));
    optimumR[second] = cross(optimumR[third], optimumR[first]);
	
	return transpose(optimumR);

}


//--------------------------------------------------------------------------------------
// Geometry Shader
// Input: 1 point
// Output 7 points
// Functionality:
//         Fetch the ApqBar from ApqBarTex, fetch AqqBar from AqqBarTex, multiply to get ABar
//         then extract the left sub 3x3 matrix and find the optimum rotation, R
//         then compute (1-beta)*R + beta*Abar and output the 
//--------------------------------------------------------------------------------------
[maxvertexcount(7)]
void COMPUTE_TRANSFORM_GS( point COMPUTE_TRANSFORM_GS_INPUT input[1], inout PointStream<COMPUTE_TRANSFORM_PS_INPUT> points)
{

	// Fetch ApqBar	
	int3 fetchPos = int3(input[0].uv.xy, 0);
	float4 ApqBar0 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar1 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar2 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar3 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar4 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar5 = ApqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 ApqBar6 = ApqBarTex.Load(fetchPos);

	// Fetch AqqBar
	fetchPos = int3(input[0].uv.zw, 0);
	float4 AqqBar0 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar1 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar2 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar3 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar4 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar5 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar6 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar7 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar8 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar9 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float4 AqqBar10 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;
	float AqqBar11 = AqqBarTex.Load(fetchPos);
	fetchPos.x++;

	// Assemble ApqBar, a 3x9 matrix
	float ApqBar[3][9] = {ApqBar0, ApqBar1, ApqBar6.x, 
						  ApqBar2, ApqBar3, ApqBar6.y, 
						  ApqBar4, ApqBar5, ApqBar6.z};
					  
	// Assemble AqqBar, a 9x9 matrix
	float AqqBar[9][9] = {
	AqqBar0.xyzw, AqqBar1.xyzw, AqqBar2.x,
	AqqBar0.y, AqqBar2.yzw, AqqBar3.xyzw, AqqBar4.x,
	AqqBar0.z, AqqBar2.z, AqqBar4.yzw, AqqBar5.xyzw,
	AqqBar0.w, AqqBar2.w, AqqBar4.z, AqqBar6.xyzw, AqqBar7.xy,
	AqqBar1.x, AqqBar3.x, AqqBar4.w, AqqBar6.y, AqqBar7.zw, AqqBar8.xyz,
	AqqBar1.y, AqqBar3.y, AqqBar5.x, AqqBar6.z, AqqBar7.w, AqqBar8.w, AqqBar9.xyz,
	AqqBar1.z, AqqBar3.z, AqqBar5.y, AqqBar6.w, AqqBar8.x, AqqBar9.x, AqqBar9.w, AqqBar10.xy,
	AqqBar1.w, AqqBar3.w, AqqBar5.z, AqqBar7.x, AqqBar8.y, AqqBar9.y, AqqBar10.x, AqqBar10.zw,
	AqqBar2.x, AqqBar4.x, AqqBar5.w, AqqBar7.y, AqqBar8.z, AqqBar9.z, AqqBar10.y, AqqBar10.w, AqqBar11.x};
						  						  
	float oneMinusBeta = 1.0f - input[0].beta;
	
	float4 transform0, 
		   transform1, 
		   transform2, 
		   transform3, 
		   transform4, 
		   transform5, 
		   transform6; 
		   
	// Compute the transformation 
       
	// Compute the transformation 
	
#define Set(Q, i, j) Q = input[0].beta*(ApqBar[i][0]*AqqBar[0][j]+ApqBar[i][1]*AqqBar[1][j]+ApqBar[i][2]*AqqBar[2][j]+ApqBar[i][3]*AqqBar[3][j]+ApqBar[i][4]*AqqBar[4][j]+ApqBar[i][5]*AqqBar[5][j]+ApqBar[i][6]*AqqBar[6][j]+ApqBar[i][7]*AqqBar[7][j]+ApqBar[i][8]*AqqBar[8][j])
#define SetNoMul(Q, i, j) Q = (ApqBar[i][0]*AqqBar[0][j]+ApqBar[i][1]*AqqBar[1][j]+ApqBar[i][2]*AqqBar[2][j]+ApqBar[i][3]*AqqBar[3][j]+ApqBar[i][4]*AqqBar[4][j]+ApqBar[i][5]*AqqBar[5][j]+ApqBar[i][6]*AqqBar[6][j]+ApqBar[i][7]*AqqBar[7][j]+ApqBar[i][8]*AqqBar[8][j])
	
	// We compute the first 3 columns of the product ApqBar*AqqBar, with SetNoMul
	// This is because we want to compute optimum rotation and blend it with A
	
	// For the rest of the columns, we multiply the result with beta, with Set, because we don't need to blend these with
	// the rotation
	
	// First 4 columns of the first row	
	SetNoMul(transform0.x, 0, 0);
	SetNoMul(transform0.y, 0, 1);
	SetNoMul(transform0.z, 0, 2);	
	Set(transform0.w, 0, 3);
	
	Set(transform1.x, 0, 4);
	Set(transform1.y, 0, 5);
	Set(transform1.z, 0, 6);
	Set(transform1.w, 0, 7);

	SetNoMul(transform2.x, 1, 0);
	SetNoMul(transform2.y, 1, 1);
	SetNoMul(transform2.z, 1, 2);
	Set(transform2.w, 1, 3);

	Set(transform3.x, 1, 4);
	Set(transform3.y, 1, 5);
	Set(transform3.z, 1, 6);
	Set(transform3.w, 1, 7);
	
	SetNoMul(transform4.x, 2, 0);
	SetNoMul(transform4.y, 2, 1);
	SetNoMul(transform4.z, 2, 2);
	Set(transform4.w, 2, 3);

	Set(transform5.x, 2, 4);
	Set(transform5.y, 2, 5);
	Set(transform5.z, 2, 6);
	Set(transform5.w, 2, 7);

	Set(transform6.x, 0, 8);
	Set(transform6.y, 1, 8);
	Set(transform6.z, 2, 8);	
	
	// Also, we want to try to conserve volume, so we compute the volume, which is just the determinate of the sub 3x3 matrix
	float volume = dot(transform0.xyz, cross(transform2.xyz, transform4.xyz));
	
	// Compute the 1/volume^(1/3) to be divided out of A matrix, also we clamp it so that it does not get too big
	float fraction = sign(volume) / max(1e-6, (sign(volume)*pow(abs(volume), 0.333333333333f)));
	// Compute A, which is the 3x3 left sub matrix of ABar	
	float3x3 A = {transform0.x, transform0.y, transform0.z,
	  			  transform2.x, transform2.y, transform2.z,
				  transform4.x, transform4.y, transform4.z};

					
	float3x3 R = ComputeOptimumRotation(A);
	
	// Compute the final value of the tranformation by weighted average A with R
	float factor = input[0].beta * fraction;
	
	transform0.xyz = factor*transform0.xyz + oneMinusBeta*R[0];
	transform2.xyz = factor*transform2.xyz + oneMinusBeta*R[1];
	transform4.xyz = factor*transform4.xyz + oneMinusBeta*R[2];
	
	transform6.w = 0.0f;
	
	

	// Output 7 adjacent fragments to represent the 3x9 transformation matrix
    COMPUTE_TRANSFORM_PS_INPUT output; 
    output.pos = input[0].pos;
    output.color = transform0;

	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform1;
	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform2;
	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform3;
	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform4;
	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform5;
	points.Append(output);

	output.pos.x += twoInv7S;
    output.color = transform6;
	points.Append(output);

}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Just output the pass through color
//--------------------------------------------------------------------------------------
float4 COMPUTE_TRANSFORM_PS( COMPUTE_TRANSFORM_PS_INPUT input) : SV_Target
{
	return input.color;
}

technique10 ComputeTransform
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// No blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_TRANSFORM_VS() ));
        SetGeometryShader( CompileShader( gs_4_0, COMPUTE_TRANSFORM_GS() ));
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_TRANSFORM_PS() ));
    }
}
//******************************************** Compute goal position of control points in each cluster *****************************************
// Compute cluster goal position

struct COMPUTE_CLUSTER_GOAL_POSITION_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_CLUSTER_GOAL_POSITION_GS_INPUT
{
    float4 pos : SV_POSITION;
    float4 uv : TEXCOORD0;
};
struct COMPUTE_CLUSTER_GOAL_POSITION_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 qBarPos : TEXCOORD0;
    float2 JUNK : TEXCOORD9;
    float4 cm : TEXCOORD1;
    float4 transform0 : TEXCOORD2;
    float4 transform1 : TEXCOORD3;
    float4 transform2 : TEXCOORD4;
    float4 transform3 : TEXCOORD5;
    float4 transform4 : TEXCOORD6;
    float4 transform5 : TEXCOORD7;
    float3 transform6 : TEXCOORD8;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//       Just pass through.
//       There will be points being rendered, one point per cluster.
//--------------------------------------------------------------------------------------
COMPUTE_CLUSTER_GOAL_POSITION_GS_INPUT COMPUTE_CLUSTER_GOAL_POSITION_VS( COMPUTE_CLUSTER_GOAL_POSITION_VS_INPUT input)
{
    COMPUTE_CLUSTER_GOAL_POSITION_GS_INPUT output;
    output.pos = input.pos;
    output.uv = input.uv;
    return output;
}


//--------------------------------------------------------------------------------------
// Geometry Shader
// Input: 1 point
// Output: 1 quad
// Functionality:
//           Will turn the point into a quad that covers all control points of the cluster.
//           Will start by fetching the center of mass and the transformation associated with the cluster and pass them as 
//           vertex attributes.
//			 Then output a quad that covers all the control points of the cluster.
//--------------------------------------------------------------------------------------
[maxvertexcount(4)]
void COMPUTE_CLUSTER_GOAL_POSITION_GS( point COMPUTE_CLUSTER_GOAL_POSITION_GS_INPUT input[1], inout TriangleStream<COMPUTE_CLUSTER_GOAL_POSITION_PS_INPUT> triangles)
{
	COMPUTE_CLUSTER_GOAL_POSITION_PS_INPUT output;
	output.JUNK = float2(0.0f,0.0f);
	int3 uv = int3(input[0].uv.xy, 0);
	
    // Fetch cm
    output.cm = cmTex.Load(uv);
    output.cm.xyz /= output.cm.w;
    output.cm.w = 1.0f;
    
    // Fetch transform
    uv.x *= 7;
    output.transform0 = transformTex.Load(uv);
    uv.x++;
    output.transform1 = transformTex.Load(uv);
    uv.x++;
    output.transform2 = transformTex.Load(uv);
    uv.x++;
    output.transform3 = transformTex.Load(uv);
    uv.x++;
    output.transform4 = transformTex.Load(uv);
    uv.x++;
    output.transform5 = transformTex.Load(uv);
    uv.x++;
    output.transform6 = transformTex.Load(uv);

	// Create a quad with a triangle strip
	output.pos = float4(input[0].pos.xy, 0.0f, 1.0f);    
	output.qBarPos = float2(-1.5f, input[0].uv.z);
	triangles.Append(output);
	
	output.pos = float4(input[0].pos.xw, 0.0f, 1.0f);    
	output.qBarPos = float2(-1.5f, input[0].uv.w);
	triangles.Append(output);
	
	output.pos = float4(input[0].pos.zy, 0.0f, 1.0f);    
	output.qBarPos = float2(threeXAdrW-1.5f, input[0].uv.z);
	triangles.Append(output);
	
	output.pos = float4(input[0].pos.zw, 0.0f, 1.0f);    
	output.qBarPos = float2(threeXAdrW-1.5f, input[0].uv.w);
	triangles.Append(output);	
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Fetch the qBar and then transform with the transformation
//--------------------------------------------------------------------------------------
float4 COMPUTE_CLUSTER_GOAL_POSITION_PS( COMPUTE_CLUSTER_GOAL_POSITION_PS_INPUT input) : SV_Target
{
	float3 uv = int3(ROUND(input.qBarPos), 0); // Obtain the position to fetch the qBar from
	
	// Fetch the qBar
	float4 qBar0 = qBarTex.Load(uv);
	uv.x += 1;
	float4 qBar1 = qBarTex.Load(uv);
	uv.x += 1;
	float qBar2 = qBarTex.Load(uv);
	
	// Compute T * qBar + cm
	return float4(
			dot(qBar0, input.transform0) + dot(qBar1, input.transform1) + qBar2*input.transform6.x,
			dot(qBar0, input.transform2) + dot(qBar1, input.transform3) + qBar2*input.transform6.y,
			dot(qBar0, input.transform4) + dot(qBar1, input.transform5) + qBar2*input.transform6.z,			
			0.0f) + input.cm;
}

technique10 ComputeClusterGoalPosition
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );   
		    
		// No Blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_CLUSTER_GOAL_POSITION_VS() ));
        SetGeometryShader( CompileShader( gs_4_0, COMPUTE_CLUSTER_GOAL_POSITION_GS() ));
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_CLUSTER_GOAL_POSITION_PS() ));
    }
}

//******************************************** Compute the goal position for control points ****************************************
// Compute g for all control points
struct COMPUTE_GOAL_POS_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_GOAL_POS_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//			 Just pass through
//--------------------------------------------------------------------------------------
COMPUTE_GOAL_POS_PS_INPUT COMPUTE_GOAL_POS_VS( COMPUTE_XVAL_VS_INPUT input)
{
    COMPUTE_GOAL_POS_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}


//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Fetch the goal position of a control point from a cluster
//           Output the position, and also set the alpha component to 1.0f
//           So we know the number clusters that influence this control point
//           Additive alpha blending is used so that we can add up the goal position 
//           contribution from all the clusters that influnce it. Later on,
//           we divide this position by the alpha (# of influnce cluster) so
//           we can get the average goal position of this control point.
//--------------------------------------------------------------------------------------
void COMPUTE_GOAL_POS_PS( COMPUTE_GOAL_POS_PS_INPUT input, out float4 g : SV_Target0)
{
	int2 uvi2 = gAdrTex.Load(int3(ROUND(input.uv),0));
	int3 uvi = int3( uvi2, 0);
    g = float4(gValTex.Load(uvi).xyz, 1.0f);
}
technique10 ComputeGoalPos
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Additive alpha blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_GOAL_POS_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_GOAL_POS_PS() ) );
    }
}
//******************************************** Compute Force for all the control points ****************************************
// Compute force for all control points
struct COMPUTE_FORCE_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_FORCE_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Just pass through
//--------------------------------------------------------------------------------------
COMPUTE_FORCE_PS_INPUT COMPUTE_FORCE_VS( COMPUTE_XVAL_VS_INPUT input)
{
    COMPUTE_FORCE_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Fetch the force acting on a control point of a cluster
//           Similar to computing goal position of a control point, we sum up the forces
//			 that act on a control point. In this case these forces come from collision
//           detection and response (because we do the collision cluster by cluster)
//--------------------------------------------------------------------------------------
void COMPUTE_FORCE_PS( COMPUTE_GOAL_POS_PS_INPUT input, out float4 f : SV_Target0)
{
	int2 uvi2 = gAdrTex.Load(int3(ROUND(input.uv),0));
	int3 uvi = int3( uvi2, 0);
    f = float4(forceValTex.Load(uvi).xyz, 1.0f);
}

technique10 ComputeForce
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    

		// Additive alpha blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_FORCE_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_FORCE_PS() ) );
    }
}
//******************************************** Pass Compute Xtilda, External acceleration ****************************************
struct COMPUTE_XTILDA_AND_ACC_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_XTILDA_AND_ACC_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//			 Just pass through
//--------------------------------------------------------------------------------------
COMPUTE_XTILDA_AND_ACC_PS_INPUT COMPUTE_XTILDA_AND_ACC_VS( COMPUTE_XTILDA_AND_ACC_VS_INPUT input)
{
    COMPUTE_XTILDA_AND_ACC_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Check if a control point penetrate through a height map or not, if so,
//           apply force in the direction of height map normal to try to push the control 
//			 point outward. Also add the force due to gravity and the force from collision
//			 Also, it will compute intermediate velocity and intermediate position
//--------------------------------------------------------------------------------------
void COMPUTE_XTILDA_AND_ACC_PS( COMPUTE_XTILDA_AND_ACC_PS_INPUT input, out float4 xTilda : SV_Target0, out float4 aTilda : SV_Target1)
{
	// Figure out where to fetch the position, velocity, and force from
	int3 fetchPos = int3(ROUND(input.uv),0);
	
	// Fetch position, velocity, and force
	float4 xn = xTex.Load(fetchPos);
	float4 vn = vTex.Load(fetchPos);
	float4 force = forceTex.Load(fetchPos);
	
	// Compute acceleration, from gravity and collision force
	float clampMass = max(xn.a, 1e-6);
	float invClampMass = 1.0f / clampMass;
	aTilda = float4(0.0f, -gravity , 0.0f, 0.0f)+any(xn.a)*force*invClampMass;
	
	// Fetch the heightmap
	float4 hmap = heightMapTex.Sample(samLinearBorder, float2(xn.x*invHeightScale.x, xn.z*invHeightScale.z)+float2(0.5,0.5f));	
	
	// Add a fudge factor
	hmap.w += HEIGHTMAP_FUDGE_FACTOR;
	
	// See if the control point penetrate the floor or not
	if (hmap.w > xn.y) {
		// If so, add force in the direction of the normal of the heightmap with magnitude proportional to the
		// height difference
		// Not exactly correct, but could be fine
		aTilda.xyz += floorHardness*(hmap.w-xn.y)*(hmap.xyz);		
	}
	
	float magAsq = dot(aTilda.xyz, aTilda.xyz);
	if (magAsq > CLAMP_ACCELERATION_SQ) {
		aTilda.xyz *= CLAMP_ACCELERATION_SQ / magAsq;
	}
	
	// Compute intermediate velocity
	float4 vTilda = (vn + h*aTilda)*any(vn.w)*airDrag;

	// Compute intermediate position
	xTilda = xn + h*vTilda;
		
	// Forward the mass 
	xTilda.w = xn.w;
}
technique10 ComputeXTildaAndAcc
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    
		
		// No blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_XTILDA_AND_ACC_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_XTILDA_AND_ACC_PS() ) );
    }
}

//******************************************** Update the velocity and position ********************************
// Update velocity and position
struct UPDATE_VEL_AND_POS_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct UPDATE_VEL_AND_POS_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Just pass through
//--------------------------------------------------------------------------------------
UPDATE_VEL_AND_POS_PS_INPUT UPDATE_VEL_AND_POS_VS( UPDATE_VEL_AND_POS_VS_INPUT input)
{
    UPDATE_VEL_AND_POS_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//         Fetch current velocity, position, intermediate position, acceleration,
//         goal position and compute the position for the next time step.
//--------------------------------------------------------------------------------------
void UPDATE_VEL_AND_POS_PS( UPDATE_VEL_AND_POS_PS_INPUT input, out float4 xnp1 : SV_Target0, out float4 vnp1 : SV_Target1)
{
	int3 fetchPos = int3(ROUND(input.uv),0);
	
	//Fetch current velocity, position, intermediate position, acceleration, goal position respectively
	float4 vn = vTex.Load(fetchPos);
	float4 xn = xTex.Load(fetchPos);
	float4 xTilda = xTildaTex.Load(fetchPos);
	float4 aTilda = aTildaTex.Load(fetchPos);
	
	float4 g = gTex.Load(fetchPos);	
	
	// Divide the goal position by the number of influence clusters
	g.xyz /= g.w;

	// Compute next step velocity
	vnp1 = (vn + aTilda*h + (g-xTilda)*vn.w*invH)*any(vn.w)*airDrag;
	// Compute next step position
	xnp1 = xn + vnp1*h;
	// Forward mass to the next time step
	xnp1.w = xn.w;
		
	// Forward alpha to the next time step
	vnp1.w = vn.w;
	
}

technique10 UpdateVelAndPos
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    
		
		// No blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
        SetVertexShader( CompileShader( vs_4_0, UPDATE_VEL_AND_POS_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, UPDATE_VEL_AND_POS_PS() ) );
    }
}

//******************************************** Update Alpha ********************************
// Update the alpha value
struct UPDATE_ALPHA_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct UPDATE_ALPHA_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Just pass through
//--------------------------------------------------------------------------------------
UPDATE_ALPHA_PS_INPUT UPDATE_ALPHA_VS( UPDATE_ALPHA_VS_INPUT input)
{
    UPDATE_VEL_AND_POS_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//         Fetch current velocity, position, intermediate position, acceleration,
//         goal position and compute the position for the next time step.
//--------------------------------------------------------------------------------------
float4 UPDATE_ALPHA_PS( UPDATE_ALPHA_PS_INPUT input) : SV_Target0
{
	int3 fetchPos = int3(ROUND(input.uv),0);
	uint objID = objIDTex.Load(fetchPos);
	float oldAlpha = vTex.Load(fetchPos).w;
	
	// If constrained, must not update
	return float4(0.0f, 0.0f, 0.0f, alphas[objID]*any(oldAlpha));
	
	//return float4(0.0f, 0.0f, 0.0f, 0.0f);
}

technique10 UpdateAlpha
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    
		
		// No blending
		SetBlendState(writeAlphaOnly, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		//SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, UPDATE_ALPHA_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, UPDATE_ALPHA_PS() ) );
    }
}
//******************************************** Compute the position of the surface mesh vertices ****************************************
// Compute the real position (position of the surface mesh vertices)
struct COMPUTE_REAL_POS_VS_INPUT
{
    float4 pos : POSITION;
    float4 uv : TEXCOORD0;
};

struct COMPUTE_REAL_POS_PS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//			 Just pass through
//--------------------------------------------------------------------------------------
COMPUTE_REAL_POS_PS_INPUT COMPUTE_REAL_POS_VS( COMPUTE_REAL_POS_VS_INPUT input)
{
    UPDATE_VEL_AND_POS_PS_INPUT output;
    output.pos = float4(input.pos.xyz, 1);
    output.uv = input.uv.xy;
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Fetch the positions of the 4 control points
//           Fetch the weight associate with the 4 control points
//           Interpolate the position
//--------------------------------------------------------------------------------------
void COMPUTE_REAL_POS_PS( COMPUTE_REAL_POS_PS_INPUT input, out float4 realPos : SV_Target0)
{
	int3 fetchPos = int3(ROUND(input.uv),0);
	uint4 posAdr = posAdrTex.Load(fetchPos);
	
	// Fetch the barycentric coordinate
	float4 posWeight = posWeightTex.Load(fetchPos);
	
	// Fetch the 4 control points that influence this vertex and weighted them appropriately
	realPos = xTex.Load(int3(posAdr.x & 65535, posAdr.x >> 16, 0)) * posWeight.x;
	realPos += xTex.Load(int3(posAdr.y & 65535, posAdr.y >> 16, 0)) * posWeight.y;
	realPos += xTex.Load(int3(posAdr.z & 65535, posAdr.z >> 16, 0)) * posWeight.z;
	realPos += xTex.Load(int3(posAdr.w & 65535, posAdr.w >> 16, 0)) * posWeight.w;	
}

technique10 ComputeRealPos
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );    
		
		// No blending
		SetBlendState(blendNone, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_REAL_POS_VS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_REAL_POS_PS() ) );
    }
}


//******************************************** Compute the normal of all surface mesh' vertices ****************************************
// Compute normal
struct COMPUTE_NORMAL_GS_INPUT
{
    float4 pos : SV_POSITION;
    float2 uv : TEXCOORD0;
};
struct COMPUTE_NORMAL_PS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Fetch real vertex's position, also compute the address that its normal 
//			 should be written to 
//--------------------------------------------------------------------------------------
COMPUTE_NORMAL_GS_INPUT COMPUTE_NORMAL_VS( uint id:SV_VertexID )
{
    COMPUTE_NORMAL_GS_INPUT output = (COMPUTE_NORMAL_GS_INPUT)0;
    
    // Fetch the position
    id += startVertex;
    int3 fetchPos = int3(id & and_posTexW, id >> shift_posTexW, 0);
    output.pos = posTex.Load(fetchPos);
    
    // Compute the address that its normal should be written to
    output.uv = float2((fetchPos.x+0.5f)*computeNormalScaleU, (fetchPos.y+0.5f)*computeNormalScaleV) - float2(1.0f, 1.0f);
    output.uv.y *= -1;
    
    return output;
}

//--------------------------------------------------------------------------------------
// Geometry Shader
// Input: 1 triangle
// Output: 3 points
// Functionality:
//           Will compute the area weighted normal of the triangle and then output the 
//		     normal to the address of the 3 vertices. Essentially, we sum up the normal
//           of the triangles that are the one-ring neighbors of each vertex
//           We can then normalize this accumulated normal afterward to get the vertex 
//           normal.
//--------------------------------------------------------------------------------------
[maxvertexcount(3)]
void COMPUTE_NORMAL_GS( triangle COMPUTE_NORMAL_GS_INPUT input[3], inout PointStream<COMPUTE_NORMAL_PS_INPUT> points )
{
    COMPUTE_NORMAL_PS_INPUT output;
 
	// Compute area weighted triangle normal
    float3 faceEdgeA = input[1].pos - input[0].pos;
    float3 faceEdgeB = input[2].pos - input[0].pos;
    float3 faceNormal = cross(faceEdgeA, faceEdgeB);
    
    // Set the color to be the normal
    output.color = float4(faceNormal,0);

	// Set the w component of the output position to 1.0f
    output.pos.w = 1.0f;
    for( int v=0; v<3; v++ )
    {
		// Set the output position to be that of the v_th vertex
        output.pos.xyz = float3(input[v].uv, 0.0f);      
        
        // Append to the output point stream
        points.Append( output );
    }
}

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//           Just pass through color
//--------------------------------------------------------------------------------------
float4 COMPUTE_NORMAL_PS( COMPUTE_NORMAL_PS_INPUT input) : SV_Target
{
    return input.color;
}

technique10 ComputeNormal
{
    pass P0
    {
		// No depth test
		SetDepthStencilState( noDepthTest, 0 );
		
		// Additive alpha blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render eveything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_NORMAL_VS() ) );
        SetGeometryShader( CompileShader( gs_4_0, COMPUTE_NORMAL_GS() ) );
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_NORMAL_PS() ) );
    }
}

//******************************************** Apply force to picked vertices ***************************
struct COMPUTE_PICKING_FORCE_VS_INPUT {
	uint2 adr : TEXCOORD0;
};

struct COMPUTE_PICKING_FORCE_PS_INPUT{
	float4 pos : SV_POSITION;
	float4 acc : COLOR;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Fetch the control point position and compute the force acting on it
//           by the mouse. Then compute the position in the forceTex texture that 
//           this vertex resides
//--------------------------------------------------------------------------------------

COMPUTE_PICKING_FORCE_PS_INPUT COMPUTE_PICKING_FORCE_VS(COMPUTE_PICKING_FORCE_VS_INPUT input) {
	COMPUTE_PICKING_FORCE_PS_INPUT output;	
	
	// Fetch the control point position
	float3 position = xTex.Load(uint3(input.adr, 0)).xyz;
	
	// Figure out the direction that this vertex should be pulled toward
	float3 vec = position-cameraPos;
	float3 pullDir = dot(vec, mouseRay)*mouseRay-vec;
	
	output.acc = float4(pullDir, 0.0f) * springK;	
	output.pos = float4((input.adr.x+0.5f)*twoInvXTexW-1.0f, 1.0f-(input.adr.y+0.5f)*twoInvXTexH, 0.0f, 1.0f);
	return output;
};

//--------------------------------------------------------------------------------------
// Pixel Shader
// Functionality:
//			 Just pass through the color
//--------------------------------------------------------------------------------------
float4 COMPUTE_PICKING_FORCE_PS(COMPUTE_PICKING_FORCE_PS_INPUT input) : SV_Target{
	return input.acc;
}

technique10 ComputePickingForce
{
    pass P0
    {
		// No Depth test
    	SetDepthStencilState( noDepthTest, 0 );
    	
    	// No Blending
		SetBlendState(blendAdd, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);    
		
		// Render everything
		SetRasterizerState(renderAll);    
		
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_PICKING_FORCE_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, COMPUTE_PICKING_FORCE_PS() ));
    }
}

//******************************************** Compute which vertices are picked *****************************************
struct COMPUTE_PICKED_VERTICES_GS_INPUT
{
	float4 pos: POSITION;
	uint2 adr:TEXCOORD0;
};

struct COMPUTE_PICKED_VERTICES_GS_OUTPUT
{
    uint2 adr : TEXCOORD0;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
// Functionality:
//           Fetch the vertex position and transform to the normalized coordinate
//--------------------------------------------------------------------------------------
COMPUTE_PICKED_VERTICES_GS_INPUT COMPUTE_PICKED_VERTICES_VS(uint id:SV_VertexID) {
	// Pick the control point
	COMPUTE_PICKED_VERTICES_GS_INPUT output;
	uint3 fetchPos = uint3(id & and_xTexW, id >> shift_xTexW, 0);
	output.pos = mul(float4(xTex.Load(fetchPos).xyz, 1.0f), worldViewProj);
	output.adr = fetchPos.xy;
	return output;
		
}	

//--------------------------------------------------------------------------------------
// Geometry Shader
// Functionality:
//			 Check if the vertex is inside the picking window or not, 
//           if so, output, otherwise, don't output
//--------------------------------------------------------------------------------------
[maxvertexcount(1)]
void COMPUTE_PICKED_VERTICES_GS(point COMPUTE_PICKED_VERTICES_GS_INPUT input[1], inout PointStream<COMPUTE_PICKED_VERTICES_GS_OUTPUT> pointStream) {
	float3 nc = input[0].pos.xyz / input[0].pos.w;
	COMPUTE_PICKED_VERTICES_GS_OUTPUT output;
	// Check if the vertex is in the range that I should mark as being picked
	
	if ( (nc.x >= minsPickNC.x) && (nc.x <= maxsPickNC.x) && 
		 (nc.y >= minsPickNC.y) && (nc.y <= maxsPickNC.y)) {
		 
		 output.adr = input[0].adr;
		 // Yes, output it to the stream
		 pointStream.Append(output);				 
	}	
}


// For the stream out
GeometryShader gsStreamOutComputePickedVertices = ConstructGSWithSO( CompileShader( gs_4_0,  COMPUTE_PICKED_VERTICES_GS() ), "TEXCOORD0.xy" );

technique10 ComputePickedVertices
{
    pass P0
    {
		// No pixel shader
        SetVertexShader( CompileShader( vs_4_0, COMPUTE_PICKED_VERTICES_VS() ) );
        SetGeometryShader(gsStreamOutComputePickedVertices );
        SetPixelShader( NULL);
    }
}

